from direct.gui.DirectGui import *
import direct.directbase.DirectStart
from direct.showbase.DirectObject import DirectObject
from direct.actor.Actor import Actor
from pandac.PandaModules import DirectionalLight, VBase4, Vec3
from functools import partial
from direct.interval.IntervalGlobal import *
import math
base.disableMouse ()

def accept_fn (event, function, *a, **k):
    obj = DirectObject ()
    obj.function = function
    obj.accept (event, obj.function, *a, **k)
    return obj

class Ralph (DirectObject, object):
    def __init__ (self, *a, **k):
        super (Ralph, self).__init__ (*a, **k)
        # Esta es la base que permanece a nivel del suelo
        base_node = render.attachNewNode ("ralph-base")
        ralph_node = Actor (
            'models/ralph.egg.pz',
            { 'run'  : 'models/ralph-run.egg.pz' })
        ralph_node.reparentTo (base_node)
        ralph_node.setBlend (frameBlend = True)
        self.base_node  = base_node
        self.ralph_node = ralph_node
        # Continua con el resto del grafo de escena ...
        ship_node = loader.loadModel ('models/ship.bam')
        ship_node.reparentTo (ralph_node.exposeJoint (
            None, 'modelRoot', 'RightHand'))
        ship_node.setScale (.2)

        light_node = ralph_node.attachNewNode (
            DirectionalLight ('ralph-light'))
        light_node.node ().setColor (VBase4 (1., .5, .5, 1.))
        light_node.setPos (0, 0, 10)
        light_node.lookAt (0, 0, 0)
        light_node.reparentTo (ralph_node)
        render.setLight (light_node)

        # Cargamos unos sonidos y la animacion
        self.jump_sfx      = loader.loadSfx ('sfx/jump.ogg')
        self.jump_done_sfx = loader.loadSfx ('sfx/fall.ogg')
        self.jump_anim     = Sequence (
            ralph_node.posInterval (.5, Vec3 (0, 0, 5)),
            ralph_node.posInterval (.5, Vec3 (0, 0, 0)))
        self.jump_anim.setDoneEvent ('ralph-jump-done')
        self.accept ('ralph-jump',      self.on_jump)
        self.accept ('ralph-jump-done', self.on_jump_done)
        
    def on_jump (self):
        if not self.jump_anim.isPlaying ():
            self.jump_sfx.play ()
            self.jump_anim.start ()
    def on_jump_done (self):
        self.jump_done_sfx.play ()

def circles_task (node, task):
    node.setPos (math.sin (task.time) * 10.0,
                 math.cos (task.time) * 10.0, 0)
    node.setHpr (- (task.time * 180 / math.pi - 90), 0, 0)
    return task.cont

camera.setPos (30, 0, 30)
camera.lookAt (0, 0, 0)

texto = OnscreenText (
    text      = "Bienvenido al juego de Ralf dando vueltas.",
    scale     = .07,
    mayChange = True)

# El boton ahora envia un evento ralph-jump
DirectButton (
    text      = "Salta!",
    pos       = (0, 0, -.15),
    scale     = .1,
    command   = lambda: messenger.send ('ralph-jump'))
# Forwadeamos la tecla espacio a ralph-jump
accept_fn ('space', lambda: messenger.send ('ralph-jump'))
ralph = Ralph ()
ralph.ralph_node.loop ('run')
taskMgr.add (partial (circles_task, ralph.base_node), 'c')
run ()
